/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTActor.h>
#include <RTActorClass.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTBindingEnd.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTInterfaceDescriptor.h>
#include <RTLayerConnector.h>
#include <RTRelayDescriptor.h>
#include <RTRelayPort.h>

int RTActorRef::bind_ports( RTActor * actor, int index, RTActorRef * importer )
{
    RTAssert( actor != (RTActor *)0 );

    RTActor                     * myOwner   = getOwner();
    const RTActorRef            * actorHome = actor->getReference();
    const RTActorClass          * type      = actor->getActorData()->type;
    const int                     numRelays = type->numRelays;
    const int                     numIfaces = _info->numInterfaces;
    int                           relayId;
    const RTRelayDescriptor     * relayDesc;
    RTRelayPort                 * relay;
    const RTInterfaceDescriptor * iface;

    for( relayId   = 0,
	 relayDesc = type->relays,
	 relay     = actor->relays,
	 iface     = _info->interfaces;

	 relayId < numIfaces;

	 ++relayId, ++relayDesc, ++relay, ++iface )
    {
	int count = iface->replication;

	if( count == 0 )
	    continue;

	if( relayId >= numRelays )
	{
	    // error: too few relay ports
	}
	else if( relay->reserve( actorHome, relayId, count,
				 relayDesc->replication, importer ) == 0 )
	{
	    // error: reservation failed
	}
	else
	{
	    continue;
	}

	// back-track undoing the reservations made to this point

	while( --relayId >= 0 )
	{
	    --relayDesc;
	    --relay;
	    --iface;

	    relay->unreserve( myOwner, relayId, relayDesc->replication,
			      importer );
	}
	return 0;
    }

    RTController * controller = myOwner->context();
    RTBindingEnd   end1;
    RTBindingEnd   end2;

    for( relayId   = 0,
	 relayDesc = type->relays,
	 relay     = actor->relays,
	 iface     = _info->interfaces;

	 relayId < numIfaces;

	 ++relayId, ++relayDesc, ++relay, ++iface )
    {
	int count = iface->replication;

	if( count == 0 )
	    continue;

	int relaySize = relayDesc->replication;
	int relayBase = count * index;

	for( int relayIndex = 0; relayIndex < count; ++relayIndex )
	{
	    if( relay->followIn( end1, actor, relayId,
				 relayIndex, relaySize, importer ) == 0 )
		continue;

	    if( myOwner->_followOutV( end2, getId(),
				      relayId, relayBase + relayIndex ) == 0 )
		continue;

	    RTLayerConnector::lock();
	    RTLayerConnector::bind( controller, end1, end2,
				    RTLayerConnector::CheckedAlready );
	    RTLayerConnector::unlock();
	}
    }

    return 1;
}
