/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTImportList.h>

int RTActorRef::indexOf( const RTActor * actor ) const
{
    int replication = _info->replication;
    int index       = -1;

    if( replication <= 0 )
    {
	// excluded reference
    }
    else if( _info->kind == RTComponentDescriptor::Imported )
    {
	RTImportList * entry;

	if( replication == 1 )
	{
	    entry = (RTImportList *)_contents;

	    if( entry->used.reference == this && entry->used.actor == actor )
		index = 0;
	}
	else for( entry = actor->importList; ; entry = entry->used.next )
	{
	    if( entry == (RTImportList *)0 )
		break;

	    if( entry->used.reference == this )
	    {
		RTAssert( entry->used.actor == actor );

		RTImportList::Unused * head;
		RTImportList         * first;

		head  = (RTImportList::Unused *)_contents;
		first = (RTImportList *)(void *)&head[ 1 ];

		index = entry - first; //lint !e946 !e947
		break;
	    }
	}
    }
    else
    {
	index = actor->getIndex();

	if( index < 0 || index >= replication || (*this)[ index ] != actor )
	    index = -1;
    }

    return index;
}
