/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTAsciiDecoding.h>

int RTAsciiDecoding::get_long( long & value )
{
    const long max_div_10 =              ( ~0UL >> 1 )   / 10UL;
    const long max_mod_10 =              ( ~0UL >> 1 )   % 10UL;
    const long min_div_10 = - (long)( ( ~( ~0UL >> 1 ) ) / 10UL );
    const long min_mod_10 = - (long)( ( ~( ~0UL >> 1 ) ) % 10UL );

    enum { someDigits = 1, overflow = 2 };

    long tmp    = 0L;
    int  status = 0;
    bool pos    = true;
    char byte;

    if( getNonWhite( byte ) == 0 )
	return 0;

    if( byte == '-' )
    {
	pos = false;
	if( getByte( byte ) == 0 )
	    return 0;
    }

    do
    {
	if( '0' <= byte && byte <= '9' )
	{
	    status |= someDigits;

	    long digit = (long)( byte - '0' );

	    if( pos )
	    {
		if( tmp < max_div_10 )
		{
		    // no possibility of overflow
		}
		else if( tmp > max_div_10 || digit > max_mod_10 )
		{
		    status |= overflow;
		}
	    }
	    else
	    {
		digit = - digit;

		if( tmp > min_div_10 )
		{
		    // no possibility of overflow
		}
		else if( tmp < min_div_10 || digit < min_mod_10 )
		{
		    status |= overflow;
		}
	    }

	    tmp *= 10L;
	    tmp += digit;
	}
	else
	{
	    ungetByte( byte );
	    break;
	}
    }
    while( getByte( byte ) != 0 );

    if( status != someDigits )
	return 0;

    value = tmp;

    return 1;
}
