/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTCmdLineObserver.h>
#include <RTDebugger.h>
#include <RTMemoryUtil.h>

static const char prompt[] = "\r\nRTS Debug ->";

int RTCmdLineObserver::receiveData( void )
{
    bool needPrompt = false;
    int  bufIndex   = 0;

    for(;;)
    {
	if( bufIndex >= bufUsed )
	{
	    int numRead = readData( &inBuffer[ bufUsed ], bufSize - bufUsed );

	    if( numRead <= 0 )
		return numRead;

	    bufUsed += numRead;
	}

	++bufIndex;

	char next = ( bufIndex < bufUsed ) ? inBuffer[ bufIndex ]
					   : '\0';

	if( inBuffer[ bufIndex - 1 ] == '\n' )
	{
	    needPrompt = true;

	    // Change each carriage-return that immediately precedes
	    // a new-line into a space. This avoids the problem of
	    // generally handling CRs elsewhere.

	    if( bufIndex > 1 && inBuffer[ bufIndex - 2 ] == '\r' )
		inBuffer[ bufIndex - 2 ] = ' ';
	}
	else if( inBuffer[ bufIndex - 1 ] == ';' )
	    inBuffer[ bufIndex -1 ] = '\n';
	else
	    continue;

	inBuffer[ bufIndex ] = '\0';

	debugger->debugIt( inBuffer );

	inBuffer[ bufIndex ] = next;

	if( (bufUsed -= bufIndex) > 0 )
	    RTMemoryUtil::memcpy( inBuffer, &inBuffer[ bufIndex ], bufUsed );

	if( ! needPrompt )
	{
	    // prompt not needed
	}
	else if( sendData( prompt, RTMemoryUtil::strlen( prompt ) ) != 0 )
	{
	    needPrompt = false;
	}

	bufIndex = 0;
    }
}
