/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTActor.h>
#include <RTController.h>
#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

void RTDebugger::step( unsigned count )
{
#if USE_THREADS
    _mutex->enter();
#endif

    mode       = Stepping;
    step_count = count;

#if OBSERVABLE
    int active_count = 0;

    if( step_count > 0U )
	evtRtsStatus();
#endif

    RTController * task = tasks;

    if( task != (RTController *)0 )
    {
	do
	{
	    switch( task->_state )
	    {
	    case RTController::waitBegin:
	    case RTController::waitContinue:
		if( step_count == 0U )
		{
		    // this is really a 'stop' command
		}
		else
		{
		    --step_count;
		    task->_state = RTController::waitDone;
#if USE_THREADS
		    wakeup( task );
#endif
#if OBSERVABLE
		    ++active_count;
#endif
		}
		break;

	    case RTController::waitDone:
	    case RTController::execUser:
#if OBSERVABLE
		++active_count;
#endif
		break;

#ifdef _lint
	    case RTController::detached:
	    case RTController::execSystem:
#endif
	    default:
		break;
	    }
	}
	while( (task = task->_next) != tasks );
    }

#if OBSERVABLE
    if( step_count == 0U && active_count == 0 )
    {
	mode = Halted;
	evtRtsStatus();
    }
#endif

#if USE_THREADS
    _mutex->leave();
#endif
}
