/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTFloat.h>

// Denormalized constants: Each is of the form ( m, d, b ) with
//     m =~ ( 10 ^ d ) * ( 2 ^ b )
// and a implicit decimal point before the most significant bit.
// These allow us to eliminate the one of the exponents in favour
// of the other.

static const RTFloat neg2[] =
{
    { { 0xF6, 0xC6, 0x9A, 0x72, 0xA3, 0x98,
	0x9F, 0x5B, 0x8A, 0xAD, 0x54, 0x9E },  180, -598 },
    { { 0xC9, 0x13, 0x93, 0x6D, 0xD5, 0x71,
	0xC8, 0x4C, 0x03, 0xBC, 0x3A, 0x1A },   89, -296 },
    { { 0xE5, 0x96, 0xB7, 0xB0, 0xC6, 0x43,
	0xC7, 0x19, 0x6D, 0x9C, 0xCD, 0x06 },   43, -143 },
    { { 0xD8, 0xD7, 0x26, 0xB7, 0x17, 0x7A,
	0x80, 0x00, 0x00, 0x00, 0x00, 0x00 },   21,  -70 },
    { { 0xEE, 0x6B, 0x28, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    9,  -30 },
    { { 0xFA, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    3,  -10 },
    { { 0xC8, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    2,   -7 },
    { { 0xA0, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    1,   -4 },
    { { 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    0,   -1 },
    { { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    0,    0 }
};

static const RTFloat pos2[] =
{
    { { 0xCF, 0x79, 0xCC, 0x9D, 0xB9, 0x55,
	0xC2, 0xCC, 0x71, 0x82, 0x14, 0x8D }, -178,  591 },
    { { 0xCB, 0xB4, 0x1E, 0xF9, 0x79, 0x34,
	0x6B, 0xCA, 0x4F, 0x2B, 0x40, 0xA0 },  -88,  292 },
    { { 0xE4, 0x5C, 0x10, 0xC4, 0x2A, 0x2B,
	0x3B, 0x05, 0x8C, 0xB8, 0x9A, 0x7E },  -44,  146 },
    { { 0xF1, 0xC9, 0x00, 0x80, 0xBA, 0xF7,
	0x2C, 0xB1, 0x53, 0x24, 0xC6, 0x8B },  -22,   73 },
    { { 0xDB, 0xE6, 0xFE, 0xCE, 0xBD, 0xED,
	0xD5, 0xBE, 0xB5, 0x73, 0x44, 0x0E },  -10,   33 },
    { { 0xD1, 0xB7, 0x17, 0x58, 0xE2, 0x19,
	0x65, 0x2B, 0xD3, 0xC3, 0x61, 0x13 },   -4,   13 },
    { { 0xA3, 0xD7, 0x0A, 0x3D, 0x70, 0xA3,
	0xD7, 0x0A, 0x3D, 0x70, 0xA3, 0xD7 },   -2,    6 },
    { { 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC,
	0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCD },   -1,    3 },
    { { 0x66, 0x66, 0x66, 0x66, 0x66, 0x66,
	0x66, 0x66, 0x66, 0x66, 0x66, 0x66 },   -1,    2 },
    { { 0x33, 0x33, 0x33, 0x33, 0x33, 0x33,
	0x33, 0x33, 0x33, 0x33, 0x33, 0x33 },   -1,    1 },
    { { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },    0,    0 }
};

void RTFloat::normalize10( void )
{
    const RTFloat * f;

    if( exp_2 > 0 )
	for( f = pos2; f->exp_2 > 0; ++f )
	    while( exp_2 >= f->exp_2 )
		multiply( *f );

    else if( exp_2 < 0 )
	for( f = neg2; f->exp_2 < 0; ++f )
	    while( exp_2 <= f->exp_2 )
		multiply( *f );

    while( mod10() == 0 )
    {
	++exp10;
	(void)div10();
    }
}

void RTFloat::normalize2( void )
{
    // Shift mantissa until leading bit is non-zero.
    {
	int   i;
	int   j;
	dword carry;
	int   shift;

	for( i = 0; mantissa[ i ] == 0; )
	{
	    if( ++i >= mantissa_words )
	    {
		exp_2 = 0;
		exp10 = 0;
		return;
	    }
	    exp_2 -= bits_per_word;
	}

	if( i != 0 )
	{
	    for( j = 0; i < mantissa_words; ++i, ++j )
		mantissa[ j ] = mantissa[ i ];
	    for( ; j < mantissa_words; ++j )
		mantissa[ j ] = 0;
	}

	for( shift  = bits_per_word, carry = (dword)mantissa[ 0 ];
	     shift -= 1,            (carry >>= 1) != 0UL; )
	{
	    // how far do we have left to go?
	}

	if( shift != 0 )
	{
	    exp_2 -= shift;
	    for( i = mantissa_words; --i >= 0; )
	    {
		carry         += ( (dword)mantissa[ i ] ) << shift;
		mantissa[ i ]  = (word)( word_mask & (word)carry );
		carry        >>= bits_per_word;
	    }
	}
    }

    // Eliminate any decimal exponent.
    {
	const RTFloat * f;

	if( exp10 > 0 )
	    for( f = neg2; f->exp10 > 0; ++f )
		while( exp10 >= f->exp10 )
		    multiply( *f );

	else if( exp10 < 0 )
	    for( f = pos2; f->exp10 < 0; ++f )
		while( exp10 <= f->exp10 )
		    multiply( *f );
    }

    // Shift mantissa so least significant bit is non-zero.
    {
	int   i;
	int   j;
	dword carry;
	int   shift;

	// find least significant non-zero word
	for( i = mantissa_words; mantissa[ --i ] == 0; )
	    exp_2 += bits_per_word;

	if( i != mantissa_words - 1 )
	{
	    for( j = mantissa_words; i >= 0; --i )
		mantissa[ --j ] = mantissa[ i ];
	    while( --j >= 0 )
		mantissa[ j ] = 0;
	}

	for( carry = (dword)mantissa[ mantissa_words - 1 ], shift = 0;
	     ( carry & 1UL ) == 0UL;
	     carry >>= 1,                                 ++shift )
	{
	    // count trailing zero bits
	}

	if( shift != 0 )
	{
	    exp_2 += shift;
	    carry = 0UL;
	    for( i = 0; i < mantissa_words; ++i )
	    {
		carry         += (dword)mantissa[ i ];
		mantissa[ i ]  = (word)( word_mask & (word)( carry >> shift ) );
		carry        <<= bits_per_word;
	    }
	}
    }
}
