/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTMessage.h>
#include <RTProtocol.h>

int RTProtocol::End::invoke( RTMessage            * replyBuffer,
			     RTController         * invoker,
			     RTProtocol           * port,
			     int                    index,
			     int                    signal,
			     const void           * data,
			     const RTObject_class * type )
{
    RTActor             * remoteActor;
    RTController::Error   error;

    // check if port is bound
    switch( state )
    {
    default:
    case Unbound:
	error = RTController::noConnect;
	remoteActor = (RTActor *)0;
	break;

    case BoundXThread:
	error = RTController::xRtsInvoke;
	remoteActor = (RTActor *)0;
	break;

    case Bound:
	remoteActor = remote->getOwner();

	if( remoteActor->isActive() != 0 )
	    error = RTController::recursiveInvoke;

#if SUSPEND
	else if( remoteActor->isSuspended() != 0 )
	    // suspended actors effectively just defer all messages
	    error = RTController::deferInvoke;
#endif

	else
	    error = RTController::ok;
	break;
    }

    // Mark the reply as invalid. If there is a reply this will change.
    replyBuffer->setInvalid();

    RTMessage * oldReplyBuffer = invoker->setReplyBuffer( replyBuffer );
    RTMessage   msg;

    msg.setData( data, type );
    msg.setDestination( remoteActor, remote, remoteIndex );
    msg.setPriority( System );

    if( signalMap != (const int *)0 )
	msg.setSignal( signalMap[ signal - 1 ] );
    else
	msg.setSignal( signal );

#if OBSERVABLE
    invoker->debugger()->setTimestamp( &msg, port, index );

    RTActor * owner = port->getOwner();

    // the invoked message
    if( owner->probe != (RTActorProbe *)0 )
	owner->probe->evtMessageOut( &msg, RTEventInfo::SendInvoke );
#endif

    if( error != RTController::ok )
    {
	invoker->setError( error,
			   RTController::CommInvoke, port, &index );
    }
    else if( msg.deliver() == RTMessage::Rejected )
    {
	// error already set
    }
    else if( replyBuffer->isValid() == 0 )
    {
	invoker->setError( RTController::noReply,
			   RTController::CommInvoke, port, &index );
    }
    else
    {
#if OBSERVABLE
	// the reply to the invoke
	if( owner->probe != (RTActorProbe *)0 )
	{
	    replyBuffer->setDestination( owner, port, index );
	    owner->probe->evtMessageIn( replyBuffer );
	}
#endif
	invoker->restoreReplyBuffer( oldReplyBuffer );

	return 1;
    }

    invoker->restoreReplyBuffer( oldReplyBuffer );

    return 0;
}
