/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#ifdef PRAGMA
#pragma implementation "RTProtocolDescriptor.h"
#pragma implementation "RTSignalDescriptor.h"
#endif

#include <RTMemoryUtil.h>
#include <RTObject_class.h>
#include <RTProtocolDescriptor.h>
#include <RTSignalDescriptor.h>

// Test whether all signals that might be sent from tx are compatible
// with those that can be received by rx assuming the signals are sorted
// in ascending order according to their names.

static
RTProtocolDescriptor::Status
sendAll( const RTProtocolDescriptor * tx,
	 const RTProtocolDescriptor * rx )
{
    const RTProtocolDescriptor   * conjugate = tx->conjugate;
    const RTSignalDescriptor     * txSignal  = conjugate->inSignals;
    int                            txCount   = conjugate->numInSignals;
    const RTSignalDescriptor     * rxSignal  = rx->inSignals;
    int                            rxCount   = rx->numInSignals;
    RTProtocolDescriptor::Status   result    = RTProtocolDescriptor::ok;

    for( ; --txCount >= 0; ++txSignal, ++rxSignal )
    {
	for( ; ; ++rxSignal )
	{
	    if( --rxCount < 0 )
		// no more signals in rx
		return RTProtocolDescriptor::badSignal;

	    int cmp = RTMemoryUtil::strcmp( txSignal->name, rxSignal->name );

	    if( cmp > 0 )
		continue; // keep looking for the signal

	    if( cmp < 0 )
		// signals are sorted, the one we want should appear
		// before this one: it must not be there
		return RTProtocolDescriptor::badSignal;

	    // found the signal, check the type
	    if( rxSignal->type == (const RTObject_class *)0 )
	    {
		// any type is acceptable
	    }
	    else if( txSignal->type == (const RTObject_class *)0 )
	    {
		// need a specific type
		result = RTProtocolDescriptor::badType;
	    }
	    else if( txSignal->type->isKindOf( rxSignal->type ) == 0 )
	    {
		result = RTProtocolDescriptor::badType;
	    }

	    break; // type is ok
	}
    }

    return result;
}

RTProtocolDescriptor::Status
RTProtocolDescriptor::compare( const RTProtocolDescriptor * other ) const
{
    Status result;

    if( other == conjugate )
    {
	result = ok;
    }
    else if( (result = sendAll( this, other )) == badSignal )
    {
	// no point in checking the other direction
    }
    else switch( sendAll( other, this ) )
    {
	default:
	case badSignal: result = badSignal; break;
	case badType:   result = badType;   break;
	case ok:                            break;
    }

    return result;
}
