/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#ifndef __RTActorSlot_h__
#define __RTActorSlot_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

class RTActor;

// If the replication factor, n, of an optional reference satisfies
// ( 2 <= n < threshold ), we allocate n + 1 RTSmallOptional objects.
// The first one serves as the anchor for the free list.
// This depends on two things:
// 1.  int is no larger than a pointer, and
// 2.  all addresses allocated on the heap are even.

struct RTSmallOptional
{
    RTS_INLINE static int supports( int size );
    RTS_INLINE void       insert( int prev, int next );
    RTS_INLINE int        inuse( void ) const;
    RTS_INLINE void       prev( int );
    RTS_INLINE int        prev( void ) const;
    RTS_INLINE void       next( int );
    RTS_INLINE int        next( void ) const;

    RTActor * _actor;

    enum
    {
	// we assume sizeof( int ) <= sizeof( RTActor * )
	link_bits  = 4 * (int)sizeof( int ) - 1
      , threshold  = 1 << link_bits
      , link_mask  = threshold - 1
      , next_shift = 1
      , prev_shift = next_shift + link_bits
    };
};

// If ( n >= threshold ), we allocate n + 1 RTLargeOptional objects.
// The first one serves as the anchor for the free list.
// The next pointer is nil when the entry has been reserved.

struct RTLargeOptional
{
    RTLargeOptional     * next;
    union
    {
	RTLargeOptional * prev;
	RTActor         * actor;
    };
};

#if RTS_INLINES
#include <RTActorSlot.inl>
#endif

#endif // __RTActorSlot_h__
