/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTDiag.h>
#include <RTSyncObject.h>
#include <RTTimespec.h>

#include <errno.h>

void RTSyncObject::timedwait( const RTTimespec & expireTime )
{
    const RTTimespec & expire = expireTime;

    enter();

    if( signalled == 0 )
    {
	// the timeout argument to pthread_cond_timedwait is 'timespec *'
	// instead of 'RTTimespec *' so we must make a copy
	timespec timeout;

	timeout.tv_sec  = expire.tv_sec;
	timeout.tv_nsec = expire.tv_nsec;

	int rc = pthread_cond_timedwait( &cv, &mutex, &timeout );

	// POSIX, it seems, is a collection of 'standards'. Some implementions
	// use the return value to indicate the reason the call failed, others
	// simply return -1 and store the reason in errno. The following works
	// with all implementations encountered thus far.

	if( rc == 0 )
	{
	    // cv was signalled
	}
	else if( rc == ETIMEDOUT || errno == ETIMEDOUT )
	{
	    // timeout expired
	}
	else if( rc == EINTR || errno == EINTR )
	{
	    // interrupted by a signal
	}
	else
	{
	    RTDiag::panic( "pthread_cond_timedwait" );
	}
    }

    signalled = 0;

    leave();
}
