/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#ifdef PRAGMA
#pragma implementation "RTMain.h"
#pragma implementation "RTMain.inl"
#pragma implementation "RTSystemDescriptor.h"
#endif

#include <RTDiag.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>
#include <RTNucleus.h>
#include <RTSystemDescriptor.h>

struct RTMainThread
{
    int     ( * entry )( void );
    NU_TASK     task;
    char        stack[ NU_MIN_STACK_SIZE ];
};

static void main_thread( UNSIGNED, void * arg )
{
    RTMainThread * info   = (RTMainThread *)arg;
    int            status;

    status = (*info->entry)();

    if( NU_Terminate_Task( NU_Current_Task_Pointer() ) != NU_SUCCESS )
	RTDiag::panic( "terminate(thread/main)" );

    // To release resources associated with this task someone must:
    //
    //  NU_Delete_Task( &info->task );
    //  delete [] (char *)info;
}

int RTMain::execute( void )
{
    int stack_size = _system->mainStackSize;

    if( stack_size < NU_MIN_STACK_SIZE )
	stack_size = NU_MIN_STACK_SIZE;

    int info_size = sizeof( RTMainThread ) - NU_MIN_STACK_SIZE + stack_size;

    RTMainThread * info = (RTMainThread *)new char [ info_size ];

#ifndef NU_NO_ERROR_CHECKING
    RTMemoryUtil::memset( &info->task, 0, (int)sizeof( info->task ) );
#endif

    info->entry = &RTMain::mainLine;

    RTNucleus::Name name;

    RTNucleus::makeName( name, RTNucleus::Thread, (const void *)0 );

    // Create what become the main thread.
    if( NU_Create_Task( &info->task,
			name,
			main_thread,
			1U, info,
			&info->stack,
			stack_size,
			(OPTION)_system->mainPriority,
			0U, // no time slice
			NU_PREEMPT,
			NU_START ) != NU_SUCCESS )
	RTDiag::panic( "create(thread/main)" );

    return 0;
}
