/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTMemoryUtil.h>
#include <RTinet.h>
#include <RTtcp.h>

extern "C" struct uport * uportlist[];

extern "C" int RTinet_bind( int                    sock,
			    const RTinet_address * address,
			    RTinet_port            port )
{
    if( port == 0 )
    {
	const RTinet_port port_min   =  1024U;
	const RTinet_port port_first = 20000U;
	const RTinet_port port_max   = 32767U;

	port = port_first;

	for( int i = 0; i < NUPORTS; ++i )
	{
	    if( uportlist[ i ] == NU_NULL )
	    {
		// skip empty slots
	    }
	    else if( uportlist[ i ]->up_lport == intswap( port ) )
	    {
		// this port number is in use try another
		if( port == port_max )
		    port = port_min;
		else
		    ++port;

		if( port == port_first )
		    return 0; // we tried them all

		i = -1;
	    }
	}
    }

    addr_struct addr;

    RTMemoryUtil::memset( &addr, 0, (int)sizeof( addr ) );

    addr.family = NU_FAMILY_IP;
    addr.port   = port;

    if( address != (const RTinet_address *)0 )
	RTMemoryUtil::memcpy( &addr.id, address, (int)sizeof( *address ) );

    return NU_Bind( sock, &addr, 0 ) >= 0;
}
