/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTSyncObject.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

// Select taskSpawn declaration which accepts a variable number of arguments.
#define __PROTOTYPE_5_0 1

#include <taskLib.h>

static void thread_run( RTJob * job )
{
    // taskSafe protects this thread from deletion
    taskSafe();

    job->mainLoop();

    // this thread is done: allow taskDelete to proceed
    taskUnsafe();

    for(;;)
	if( taskSuspend( taskIdSelf() ) != OK )
	    RTDiag::panic( "taskSuspend" );
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job ( job )
    , _info( new RTThreadInfo )
{
    int options;

    if( taskOptionsGet( taskIdSelf(), &options ) != OK )
	RTDiag::panic( "taskInfoGet" );

    else
    {
	_info->tid = taskSpawn( (char *)job->name(),
				prio,
				options & VX_FP_TASK,
				stack_size,
				(FUNCPTR)thread_run,
				(int)job );

	if( _info->tid == ERROR )
	    RTDiag::panic( "taskSpawn" );
    }
}

RTThreadInfo::RTThreadInfo( void )
    : tid( ERROR )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( tid == ERROR )
    {
	// never spawned
    }
    else if( taskDelete( tid ) != OK )
    {
	RTDiag::panic( "taskDelete" );
    }
}
