/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTDiag.h>
#include <RTSyncObject.h>
#include <RTTimespec.h>

#include <errno.h>

void RTSyncObject::timedwait( const RTTimespec & expireTime )
{
    mutex_lock( &mutex );

    if( signalled == 0 )
    {
	// because the timeout argument to cond_timedwait is 'timespec *'
	// instead of 'RTTimespec *' we make a copy
	timespec timeout;

	timeout.tv_sec  = expireTime.tv_sec;
	timeout.tv_nsec = expireTime.tv_nsec;

	int rc = cond_timedwait( &cv, &mutex, &timeout );

	if( rc == 0 )
	{
	    // signalled
	}
	else if( rc == ETIME )
	{
	    // timeout
	}
	else if( rc == EINTR )
	{
	    // interrupted
	}
	else if( rc == EAGAIN )
	{
	    // ??
	}
	else if( errno != EINTR  && // someone has signaled me
		 errno != ETIME  && // the timeout value has expired
		 errno != EAGAIN )  // ??
	{
	    RTDiag::panic( "cond_timedwait" );
	}
    }

    signalled = 0;

    mutex_unlock( &mutex );
}
