/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

static void * thread_run( void * job )
{
    ((RTJob *)job)->mainLoop();

    return (void *)0;
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job ( job )
    , _info( new RTThreadInfo )
{
    if( stack_size < 0 )
	stack_size = 0;

    unsigned ustack = (unsigned)stack_size;

    // make sure the stack is not too small
    if( ustack < thr_minstack() )
	ustack = thr_minstack();

    if( thr_create( (void *)0, ustack,
		    thread_run, job,
		    THR_BOUND, &_info->tid ) != 0 )
	RTDiag::panic( "thr_create" );
    else
	thr_setprio( _info->tid, prio );
}

RTThreadInfo::RTThreadInfo( void )
    : tid( (thread_t)0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( tid == (thread_t)0 )
    {
	// never successfully created
    }
    else if( thr_join( tid, (thread_t *)0, (void **)0 ) != 0 )
    {
	RTDiag::panic( "thr_join" );
    }
}
