#! perl -w

# Copyright (c) 1991-2003 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# This script creates a VxWorks AE component, adds all object files to it,
# and builds it.
#
# Example usage:
# ===============
# perl link.pl ARCH=ppc603 -o TopCapsule.cm \
#   main.o %OLIST user_obj.o -flag ...
#
# The general algorithm:
#   Clean up.
#   Create a new VxWorks AE component.
#   Add each model object file and the rts libraries to the component project.
#   Set project's LD flags.
#   Build the component.

$ROSERT_HOST = $ENV{'ROSERT_HOST'} || '?';

if( $ROSERT_HOST eq 'win32' )
{
    $cp_cmd = 'copy';
}
else
{
    $cp_cmd = 'cp';
}

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

sub mySystem
{
    my( $OS ) = $ENV{'OS'} || '?';

    if( $ROSERT_HOST eq 'win32' )
    {
	foreach $arg ( @_ )
	{
	    $arg =~ s/\//\\/g; # win32 likes \ better than /
	}
    }

    if( $ROSERT_HOST eq 'win32' && $OS !~ m/^Windows_NT$/i )
    {
	return system( join( ' ', @_ ) );
    }
    else
    {
	return system( @_ );
    }
}

sub _copy
{
    my( $src  ) = shift @_;
    my( $dest ) = shift @_;

    if( $result = &mySystem( "$cp_cmd \"$src\" \"$dest\"" ) )
    {
	print STDERR "link.pl: Can't copy $src to $dest.\n";
    }

    return $result;
}

sub _rmtree     # returns 0 on success, non-zero on failure
{
    my( $dead_root ) = shift @_;

    if( -d $dead_root )
    {
	my( $result ) = 0;

	opendir DIR, $dead_root;
	my( @dead_files ) = grep !/^\.\.?$/, readdir DIR;     # skip . and ..
	closedir DIR;

	foreach $dead_file ( @dead_files )
	{
	    $result += &_rmtree( $dead_root.'/'.$dead_file );
	}

	if( ! rmdir $dead_root )
	{
	    print STDERR "link.pl: Can't delete $dead_root: $!\n";
	    return 1;
	}

	return $result;
    }
    elsif( -f $dead_root )
    {
	if( ! unlink $dead_root )
	{
	    print STDERR "link.pl: Can't delete $dead_root.\n";
	    return 1;
	}

	return 0;
    }
    else
    {
	print STDERR "link.pl: Can't delete $dead_root.\n";
	return 1;
    }
}

sub report_ges_error
{
    my( $ERROR ) = @_;

    print "description '$ERROR' severity 'error'\n";
    print "result 'failure'\n";

    exit 1;
}

$arch_pattern = 'ARCH=(ppc603|x86|simsparcsolaris|simnt)';

sub show_usage
{
    &report_ges_error( "Usage: $0 $arch_pattern options ... " );
}

&show_usage() unless $#ARGV >= 1;

if( $ARGV[0] =~ m/^$arch_pattern$/ )
{
    $arch = $1;
    shift @ARGV;
}
else
{
    &show_usage();
}

$status = 0;                              # assume all is well

if( $arch eq 'ppc603' )
{
    $build_template = 'PPC603gnu.release';
}
elsif( $arch eq 'x86' )
{
    $build_template = 'I80386gnu.release';
}
elsif( $arch eq 'simsparcsolaris' )
{
    $build_template = 'SIMSPARCSOLARISgnu.release';
}
elsif( $arch eq 'simnt' )
{
    $build_template = 'SIMNTgnu.release';
}
else
{
    show_usage();
}

foreach $arg ( @ARGV )
{
    if( $arg eq '-o' )
    {
	$lasto = 1;
    }
    elsif( $lasto )
    {
	if( $arg =~ m/(.*?)\.cm/ )
	{
	    $output = $1;
	}
	else
	{
	    $output = $arg;
	}

	$lasto  = 0;
    }
    elsif( $arg =~ m/^%(.*)$/ )    # file containing the names of object files
    {
	$in = $1;
	if( ! open( IN, "<$in" ) )
	{
	    print STDERR "link.pl: Can't read '$in': $!\n";
	    $status = 1;
	}
	else
	{
	    while( $o_file = <IN> )
	    {
		chomp $o_file;
		push @ld_files, $o_file . ".o";
	    }
	}
    }
    elsif( $arg =~ m/^-/ )    # begins with '-', so it's a flag
    {
	if( $arg =~ m/^-l(.*)$/ )  # discard -l<lib> flags
	{                          # the VxWorks AE project utility ignores them
	    push @l_libs, $1;
	}
	elsif( $arg =~ m/^-L(.*)$/ )  # discard -L<lib> flags
	{                     # the VxWorks AE project utility ignores them too
	    push @l_lib_paths, $1;
	}
	else
	{
	    push @ld_flags, $arg;
	}
    }
    else
    {
	push @ld_files, $arg;
    }
}

push @ld_flags, '-X';      # discard all local symbols
			   # comes with VxWorks AE component config by default

# clean up in case a project directory already exists

_rmtree( "./$output"    ) if( -e "./$output"    );
unlink ( "./$output.cm" ) if( -e "./$output.cm" );

# Create a new VxWorks AE component project.

$prj_dir = "./$output";
$status = &mySystem( "prjCreate -type component -prjdir $prj_dir "
		   . "-buildtemplates $build_template" )
		if $status == 0;

# add libObjecTime.a and libObjecTimeTypes.a
# These two libraries are normally supplied as -l flags, but they have to be
# added as binary components, because the VxWorks AE project build utility
# ignores -l<lib> options.
# also add any other libraries that got passes with the -l flags.

foreach $lib ( @l_libs )
{
    foreach $lib_path ( @l_lib_paths )
    {
	if( -f "$lib_path".'/lib'."$lib".'.a' )
	{
	    push @ld_files, "$lib_path".'/lib'."$lib".'.a';
	    last;
	}
    }
}

# add the supplied object files

foreach $file ( @ld_files )
{
    if( $file =~ m/.*\/(.*?)$/ )
    {
	$file_name = $1;
    }
    else
    {
	$file_name = $file;
    }

    # the binary module files have to be in the extbin directory

    $status = &_copy( $file, "$prj_dir/extbin/$build_template/" );

    $status = &mySystem( "prj compBinaryModuleAdd -p $prj_dir "
			. "\"$prj_dir/extbin/$build_template/$file_name\"" )
		if $status == 0;
}

# set the LD flags

$flags = join ' ', @ld_flags;
$status = &mySystem( "prj projBuildToolFlagsSet -p $prj_dir $build_template LD "
		   . "\"$flags\"" )
		if $status == 0;

# build the thing

$status = &mySystem( "prj projBuild -p $prj_dir" )
		if $status == 0;

$status = _copy( "$prj_dir/$build_template/$output.cm", "./" );

# Keep the component project directory in case the component needs to be added
# to an application domain.

exit $status if $status < 256;
exit $status / 256;
