# Copyright (c) 1991-2003 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# These rules are intended for use by ObjecTime personnel only.

RCS_VERSION = R650C01

# ------------------------------------------------------------

prepare : FORCE
	@ test '$(CONFIG)' != MustBeSpecified \
	  || $(FEEDBACK) -fail 'Usage: make CONFIG=<config> $@'
	@ $(PERL) Build.pl $(BUILDOPTS) $(CONFIG)

# ------------------------------------------------------------

ALL_SETS = ( cd ../config; $(PERL) ../private/Supported.pl */setup.pl )

all.prepare : FORCE
	@ for set in `$(ALL_SETS)`; do		\
	    $(MAKE) prepare CONFIG=$$set;	\
	  done

# ------------------------------------------------------------

MATCHING_SETS = $(ALL_SETS) | egrep '$(PATTERN)'

some.list : FORCE
	@ test -n '$(PATTERN)' \
	    || $(FEEDBACK) -fail 'Usage: make PATTERN=<re> $@'
	@ $(MATCHING_SETS)

some.prepare : FORCE
	@ test -n '$(PATTERN)' \
	    || $(FEEDBACK) -fail 'Usage: make PATTERN=<re> $@'
	@ for set in `$(MATCHING_SETS)`; do	\
	    $(MAKE) prepare CONFIG=$$set;	\
	  done

# ------------------------------------------------------------

FIND_SRC = find * \
		-name lib       -prune \
	     -o -name status    -prune \
	     -o -name 'build-*' -prune \
	     -o -type f -size -500

labels : rcslabel rcslabel_special rcslabel_version
	@ test -n '$(RCS_LABEL)' \
	  || $(FEEDBACK) -fail 'Usage: make RCS_LABEL=<label> labels'
	./rcslabel_version "$(RCS_VERSION)" "$(RCS_LABEL)"
	./rcslabel_special "$(RCS_VERSION)" "$(RCS_LABEL)" ../RUN_ME \
							   ObjecTime.mk
	cd ..; $(FIND_SRC) ! -perm -200 -print \
	    | sort | xargs 2>&1 ./src/rcslabel "$(RCS_VERSION)" "$(RCS_LABEL)"

# ------------------------------------------------------------

src.list : FORCE
	@ cd ..; $(FIND_SRC) -print | sort \
	    | sed -e 's|^|../|' -e 's|^../src/||'

# ------------------------------------------------------------

clean : FORCE
	-rm -rf ../build-* core

spotless : clean
	-rm -rf ../lib

STATUS_AWK='\
BEGIN{s=0}\
/^RCS file:/{s=1;printf "%s:",$$3}\
/^	/{if(s)printf " %s%s",$$1,$$2}\
/^access list:/{s=0;printf "\n"}'

status :
	@ cd ../RCS;				\
	  find . -type f -print			\
	    | sed -e '/,v$$/!d' -e 's:^\./::'	\
	    | xargs rlog -L -h			\
	    | awk $(STATUS_AWK)			\
	    | sed -e 's/,v:/:/'			\
	    | sort

src.update : FORCE ../RUN_ME rcs_update
	@ for f in ../RUN_ME rcs_update; do \
	    test -w $$f \
	     || rcsdiff -q -r$(RCS_VERSION) $$f >/dev/null \
	     || $(FEEDBACK) -fail $$f is out of date; \
	  done
	@- rm -rf ../build-*/depend.mk
	@ $(FEEDBACK) 'Using "RUN_ME" to get any missing components...'
	@ cd ..; ./RUN_ME
	@ $(FEEDBACK) 'Checking that files are current...'
	@ cd ..; $(FIND_SRC) -print | sort \
	    | RCS_VERSION=$(RCS_VERSION) xargs ./src/rcs_update

report : ../private/report.pl FORCE
	@ test -n '$(RCS_SRC)' -a -n '$(RCS_TARGET)' \
	  || $(FEEDBACK) -fail 'Usage: make RCS_SRC=<label> RCS_TARGET=<label> $@'
	cd ..; $(FIND_SRC) -print | sort \
	    | xargs 2>&1 $(PERL) private/report.pl $(RCS_SRC) $(RCS_TARGET)

NCSLOPTS=-DUSE_THREADS=1

count : ncsl
	@ $(FEEDBACK) 'Using options:'
	@ $(FEEDBACK) '  $(NCSLOPTS)'
	@ $(FEEDBACK) ''
	@ cd ..; $(FIND_SRC) -name '*[ch]' -print | sort \
		| NCSLOPTS='$(NCSLOPTS)' ./src/ncsl

# Compare all files other than those in output directories.
# Diffro only compares files which are read-only making it
# useful for checking if others have modified files which
# appear within this tree.

diffro : FORCE
	@ cd ..; $(FIND_SRC) ! -perm -200 -print \
		| sort \
		| xargs 2>&1 -l1 -t rcsdiff -q -r$(RCS_VERSION) \
		| $(PERL) private/rcsdiff_filter.pl

diffrw : FORCE
	@ cd ..; $(FIND_SRC) -perm -200 -print \
		| sort | xargs 2>&1 -l1 -t rcsdiff -q -r$(RCS_VERSION)

diffall : FORCE
	@ cd ..; $(FIND_SRC) -print \
		| sort \
		| xargs 2>&1 -l1 -t rcsdiff -q -r$(RCS_VERSION) \
		| $(PERL) private/rcsdiff_filter.pl

FORCE :
	@ $(NOP)
