/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTPubl/Control.h>
#include <RTPubl/Main.h>
#include <RTPubl/SyncObj.h>

#include <RTPriv/RTstdio.h>
#include <RTPriv/SyncObj.h>

#include <errno.h>

/*lint -save -e746 */
void RTSyncObject_wait( RTSyncObject * this )
{
    if( mutex_lock( &this->mutex ) != 0 )
    {
	RTController_setError( RTMain_controller(),
			       RTController_internalError,
			       RTController_SystemService, 0, 0 );
#if STDIO_ENABLED
	RTStdio_panic( "mutex_lock" );
#endif
    }

    if( this->signalled == 0 )
    {
	int rc = cond_wait( &this->cv, &this->mutex );

	if( rc == 0 )
	{
	    /* signalled */
	}
	else if( rc == EINTR )
	{
	    /* interrupted */
	}
	else if( rc == EAGAIN )
	{
	    /* ?? */
	}
	else if( errno != EINTR &&
		 errno != EAGAIN ) /* ?? */
	{
	    RTController_setError( RTMain_controller(),
				   RTController_internalError,
				   RTController_SystemService, 0, 0 );
#if STDIO_ENABLED
	    RTStdio_panic( "cond_wait" );
#endif
	}
    }

    this->signalled = 0;

    if( mutex_unlock( &this->mutex ) != 0 )
    {
	RTController_setError( RTMain_controller(),
			       RTController_internalError,
			       RTController_SystemService, 0, 0 );
#if STDIO_ENABLED
	RTStdio_panic( "mutex_unlock" );
#endif
    }
}
/*lint -restore */
