#!/usr/local/bin/bin.sun4-Solaris/perl -w

# Copyright (c) 1991-2003 Rational Software Corporation. All rights
# reserved.
# 
# Rational, the Rational logo, ObjecTime (and logo) are registered
# trademarks of Rational Software Corporation.
# 
# The placing of the copyright notices on Rational source code is not
# intended to mean that they are published works, but are included in the
# event of inadvertent publication. The source code of Rational contains
# trade secrets and other valuable confidential information of Rational
# Software Corporation. Rational Software Corporation and its licensors
# retain ownership to Rational Products in both source and object code
# format, and related documentation and all rights therein including
# copyrights, patents and trade secrets.
# 
# 
# U.S. Government Rights
# 
# Use, duplication, or disclosure by the U.S. Government is subject to
# restrictions set forth in the applicable Rational License Agreement
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
# or FAR 52.227-14, as applicable.
# 
# Unless otherwise agreed to in writing, use of Rational products is
# governed by the Rational Product License Agreement provided with
# the product.

# This script is necessary because Tornado/Cygnus' compiler does not take
# files from an indirect file-list, and if you have a large number
# of classes you can overflow the 4K command limit on NT.
# We can get around this by breaking up long commands into smaller
# partial-link commands...
# OR, we can get around it by calling system() from Perl, which 
# will avoid calling a sub-shell and hence avoiding the 4K limit.
# (cf. Learning Perl on Win32 systems, p. 152)
#
# This script also has another purpose:
# There are several steps to perform when compiling for Tornado.
# This script enables them all to be wrapped inside a single OTLINK_CMD.
#
# Expected usage:
# ===============
# perl otlink.cmd TopActor -- \
#     perl link.pl ARCH=386 -o TopActor.exe \
#         main.o %OLIST User_obj.o -L X -l x
#
# valid values for ARCH: 386, 68k, ppc
#
# The general algorithm:
#   @obj_list = <OLIST>
#   ld386 -r -o RTmunch0.o main.o {obj_list} User_obj.o -L X -l x
#   nm386 RTmunch0.o RTmunch1.o | munch > RTxtor.c
#   cc386 -traditional -o RTxtor.o -c RTxtor.c
#   ld386 -r -o TopActor.exe RTxtor.o RTmunch0.o
#
# If paths to main.o or libraries cause problems by requiring the shell,
# it may be possible to link {obj_list} separately.


#BEGIN {
#    @INC = ();
#    unshift(@INC, ("$ENV{OBJECTIME_HOME}/perl/lib"));
#}
#use strict;

my $maxBytesPerCommand=1000000;
# we thought $maxBytesPerCommand needed to be 4096 for NT
# but apparently Perl is smart enough to circumvent the shell
# when it needs to.

# Win95 command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.


#
# This is for the cc960 compiler which inserts annoying .lf symbols
# into the code.  So we have to clean them out.
#
sub cleanLF
{
my( $infile )= @_;

if (! -r $infile) {
    die "Can't read input $infile\n";
}
if (! -f $infile) {
    die "Input $infile is not a plain file\n";
}

open(INPUT,"<$infile") ||
    die "Can't input $infile $!";

$outfile = "c" . $infile;

open(OUTPUT,">$outfile") ||
    die "Can't output $outfile $!";

while (<INPUT>) {
    if ($_ =~ /.*\.lf.*/ )		#<-- look for lines containing ".lf" 
    {
       	chop; 
    	print OUTPUT "/* ",$_," */\n";	#<-- comment out those lines
    } 
    else
    {
	print OUTPUT $_;		#<-- just print out lines without ".lf"
    }
}


close INPUT;
close OUTPUT;
}

sub mySystem
{
  my $status=0;

    my( $os ) = $ENV{'OBJECTIME_OS'};

    if( defined( $os ) && $os eq 'Windows_95' )
    {
	$status = system( join( ' ', @_ ) )
    }
    else
    {
	$status = system( @_ );
    }

    return 0 if( $status == 0 );

    if( $status < 256 )
    {
      exit $status;
    }
    else
    {
      exit $status / 256;
    }
}

sub report_ges_error
{
    my( $ERROR ) = @_;

    print STDOUT "description '$ERROR' severity 'error'\n";
    print STDOUT "result 'failure'\n";

    exit 1;
}


$#ARGV >= 0
  || &report_ges_error( "Usage: ${0} ARCH=<arch> -o <output_file> options ... " );
my $arch="Unknown";
if( $ARGV[0] =~ /ARCH=(\w*)/ )
{
    $arch=$1;
    shift @ARGV;
}

my $ld="ld$arch";
my $nm="nm$arch";
my $cc="cc$arch";

my $output_file="a.out";
if( $ARGV[0] eq "-o" )
{
  shift @ARGV;
  $output_file=shift @ARGV;
}

my $munch="munch";
my $file_suffix=".o";

# find, remove and expand all indirect object file lists (identified by %)

my @filelist=();
my @new_argv=();

my $arg;
foreach $arg ( @ARGV )
{
    if( $arg =~ m{^%(.*)$} )
    {
	my $listfile  = $1;
	if( ! open( IN, "<$listfile" ) )
	{
	    &report_ges_error("Cannot read [$listfile]: $!");
	}
	else
	{
	    @filelist=<IN>;
	    close( IN );
	    chomp @filelist;
	    my $file;
	    foreach $file ( @filelist )
	    {
		$file .= $file_suffix;
	    }
	}
    }
    else
    {
	push @new_argv, $arg;
    }
}


my $cMunch=0;

#
# make one big object file to munch
#
&mySystem("$ld -r -o RTmunch0.o @filelist @new_argv");
my @munch_list=("RTmunch0.o");

my $cleanedfile="RTxtor.c";

#
# munch the big objetc file dumping the output to $cleanedfile
#
&mySystem("$nm @munch_list | $munch > $cleanedfile");

#
# if we're compiling for the i960 clean out the .lf symbols
#
if ( $arch =~ "960" ){
	&cleanLF("$cleanedfile");
	$cleanedfile="cRTxtor.c";
}

&mySystem("$cc -traditional -o RTxtor.o  -c $cleanedfile");
&mySystem("$ld -r -o $output_file RTxtor.o @munch_list");
