#!/bin/sh

# Location of your ClearCase installation
ATRIA=${ATRIAHOME:-/usr/atria}

# Location of your ClearCase commands
ATRIABIN=$ATRIA/bin

# Location for element type icons and magic for the menu operation
ICONDIR=$ATRIA/config/ui/icon
BITMAPDIR=$ATRIA/config/ui/bitmaps
MAGICDIR=$ATRIA/config/magic

unittypename=rosert_unit
manager=petalrt_file_delta
basemanager="text_file_delta"
basetype="text_file"

# The two icon files $unittypename.40 and $unittypename_selected.40 must be
# in the same directory as this script.

# To setup "$unittypename" as a ClearCase element type, the 'install' operation
# defined in this script must be executed.

# Init sets up "$unittypename" as an element type by defining
# $HOME/.icon/$unittypename.icon, $HOME/.magic/cc_$unittypename.magic,
# $HOME/.bitmaps/$unittypename.40 and $HOME/.bitmaps/$unittypename_selected.40

# This script also defines an "unistall" command, for destroying the items
# created by the "init" command.

copy_file()
{
    if [ ! -d $the_dir ]; then
        mkdir $the_dir >> $log 2>>$err
    fi

    if [ ! -f $the_dir/$the_file ]; then
        cp $scriptdir/$the_file $the_dir >> $log 2>> $err
        didit=1
    fi
}

makeeltype()
{
	if [ "$vob" = "" ]; then
		echo "A vob must be specified to register the element type."
	else
		eldesc=$unittypename@vob:$vob

		if cleartool lstype eltype:$eldesc >> /dev/null 2>> /dev/null; then
			replace="-replace "
		else
			replace=" "
		fi

		if cleartool mkeltype $replace -global -supertype $basetype -c "RoseRT model unit, Version 1.0" -manager $manager $eldesc >> $log 2>> $err; then
			echo \+\+\+ RoseRT units may now be managed in VOB "$vob" >> $log
		fi
	fi
}

setup_unit()
{
   model_integrator_operations="compare merge xcompare xmerge"
   install_mi_operations=
   mi_operations_already_installed=

    if [ ! -d $ATRIAMGRS/$manager ]; then
		# Need to create the manager directory in $ATRIAMGRS
		if mkdir $ATRIAMGRS/$manager >> $log 2>> $err; then
            basemanager_files="`ls $ATRIAMGRS/$basemanager`"
            for files in $basemanager_files
            do
              if [ "$files" != "compare" ] && [ "$files" != "xcompare" ] && [ "$files" != "merge" ] && [ "$files" != "xmerge" ]
              then
                if ln -s $ATRIAMGRS/$basemanager/$files $ATRIAMGRS/$manager/$files > /dev/null 2>&1; then 
                  link_success=1
                else
                  link_success=
                fi
              fi
            done

            #link the model integrator operations
            for files in $model_integrator_operations
            do
              if ln -s $ROSERT_HOME/bin/RoseRT $ATRIAMGRS/$manager/$files > /dev/null 2>&1; then 
				link_success=1
              else
                link_success=
              fi
            done
        else
          link_success= 
        fi
    else
      #If "$manager" directory exists, make sure modelint operations in place
      install_mi_operations=1
      
      if [ -z "$FORCE_MODELINT_CHANGE" ]; then
        for files in $model_integrator_operations
		do
          if [ -f "$ATRIAMGRS/$manager/$files" ]; then
			mi_operations_already_installed=1
            unset install_mi_operations
          else
            need_to_reinstall_mi_operations=1 
          fi
        done
      else
        need_to_reinstall_mi_operations=1
      fi
   
      if [ -n "$need_to_reinstall_mi_operations" ]; then
        reinstall_mi_operations=1  
        for files in $model_integrator_operations 
        do
          if [ -h "$ATRIAMGRS/$manager/$files" ]; then
            if rm -f $ATRIAMGRS/$manager/$files > /dev/null 2>&1; then
              unset install_mi_operations
            else
              mi_file_removal_problem=1
            fi
          fi

          if ln -s $ROSERT_HOME/bin/RoseRT $ATRIAMGRS/$manager/$files > /dev/null 2>&1; then 
            link_success=1
            unset install_mi_operations
          else
            link_success=
          fi
        done
       fi
     fi

    if [ -n "$link_success" ] && [ -z "$mi_file_removal_problem" ]
    then
       if [ -z "$reinstall_mi_operations" ]
       then
         echo \+\+\+ The type manager for $unittypename has been added to the ClearCase server on "${HOST:-this machine}">> $log
       else
         echo \+\+\+ The type manager for $unittypename has been reinstalled in the ClearCase server on "${HOST:-this machine}">> $log
       fi
    else
       if [ -n "$mi_operations_already_installed" ]
       then
         echo \+\+\+ The type manager for $unittypename has already been added to the ClearCase server on "${HOST:-this machine}">> $log
         else if [ -n "$install_mi_operations" ]
         then
           echo \*\*\* "Can't install Model Integrator operations on "${HOST:-"this machine"}". Greater access to $ATRIAMGRS may be required." >> $err
         else
           echo \*\*\* "Can't create the $unittypename type manager on "${HOST:-"this machine"}". Greater access to $ATRIAMGRS may be required." >> $err
         fi
       fi
   fi
    
    if [ ! -s $err ]; then
        if [ $insrvr -lt 2 ]; then
      	    scriptdir=$1
	        didit=0
    	    the_dir=$ICONDIR
    	    the_file=$unittypename.icon
			copy_file

    	    the_dir=$BITMAPDIR
    	    the_file=$unittypename.40
   	        the_line=""
    	    copy_file

			# Same params as before, but different file
			the_file=${unittypename}_selected.40
			copy_file

    	    the_dir=$MAGICDIR
    	    the_file=cc_$unittypename.magic
    	    copy_file

	      if [ $didit -eq 1 ] && [ ! -s $err ]; then
	        echo \+\+\+ Icon and file typing rules for $unittypename have been installed in $MAGICDIR, $ICONDIR, and $BITMAPDIR >> $log
	      fi
        fi
    fi
}

show_usage()
{
	echo ""
	scriptname=`basename $0`
	echo "$scriptname install [-eltype -vob <vobname> | [ -user | -server ] ]
		   [-icon <icondir>] [-bitmap <bitmapdir>] [-magic <magicdir>]"

	echo ""
	echo "Example:  $scriptname install -server"
	echo ""
}


PATH=/usr/openwin/bin:$ATRIABIN:/usr/bin:/bin
ATRIAMGRS=$ATRIA/lib/mgrs
log=/tmp/logCC$$
err=/tmp/errCC$$
command=$1
mydir="$ROSERT_HOME/bin/$ROSERT_HOST/cc"
vob=

if [ "$#" = 0 ]; then
  show_usage
  exit 1
fi

if [ -z "$ROSERT_HOME" ] || [ -z "$ROSERT_HOST" ]; then
	echo "The ROSERT_HOME and ROSERT_HOST environment variables must be defined."
	exit 1
fi

if [ "$command" = "install" ] || [ "$command" = "uninstall" ] ; then
    insrvr=2
    rmmgr=2
	mkel=0
    shift
    
    while [ $# -gt 0 ]; do
        case $1 in
    	    -server)
				ICONDIR=$ATRIA/config/ui/icon
				BITMAPDIR=$ATRIA/config/ui/bitmaps
				MAGICDIR=$ATRIA/config/magic
				insrvr=1
				shift
				;;
    	    -user)
				ICONDIR=$HOME/.icon
				BITMAPDIR=$HOME/.bitmaps
				MAGICDIR=$HOME/.magic
				insrvr=0
				shift
				;;
    	    -icon)
				ICONDIR=$2
				insrvr=0
				shift
				shift
				;;
			-bitmaps | -bitmap)
				BITMAPDIR=$2
				insrvr=0
				shift
				shift
				;;
    	    -magic)
				MAGICDIR=$2
				insrvr=0
				shift
				shift
				;;
            -manager)
                rmmgr=1
                shift
                ;;
            -modelint_change)
                FORCE_MODELINT_CHANGE=1
                insrvr=4
                shift
                ;;
			-eltype)
				mkel=1
				shift
				;;
			-vob)
				vob=$2
				shift
				shift
				;;
             -*)
                echo Unrecognized option $1
                exit
                ;;
              *)
                break
                ;;      
    	esac
    done

	if [ "$command" = "install" ]; then
		if [ "$mkel" -eq "1" ]; then
			makeeltype
		else
			setup_unit "$mydir" $*
		fi
	elif [ "$command" = "uninstall" ]; then
		if [ $insrvr -lt 2 ]; then
			echo ::: Removing customizations for element type "$unittypename" from $ICONDIR, $BITMAPDIR, and $MAGICDIR
		
			rm $ICONDIR/$unittypename.icon
			rm $BITMAPDIR/$unittypename.40
			rm $BITMAPDIR/${unittypename}_selected.40

			if [ "$ICONDIR" = "$HOME/.icon" ]; then
				rmdir $ICONDIR
			fi

			if [ "$BITMAPDIR" = "$HOME/.bitmaps" ]; then
				rmdir $BITMAPDIR
			fi
		
			rm $MAGICDIR/cc_$unittypename.magic

			if [ "$MAGICDIR" = "$HOME/.magic" ]; then
				rmdir $MAGICDIR
			fi
		fi

		if [ $rmmgr -eq 1 ]; then
			echo ::: Removing $manager manager from "${HOST:-this server}"
			rm -rf $ATRIAMGRS/$manager
		fi

		exit
	fi
else
	echo "Unrecognized command $command."
fi

if [ -s $err ]; then
    cat $err
fi
